from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.health_response_status import HealthResponseStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.details import Details


T = TypeVar("T", bound="HealthResponse")


@_attrs_define
class HealthResponse:
    """Health check response.

    Attributes:
        policies_loaded (bool):
        status (HealthResponseStatus):
        version (str):
        details (Details | Unset):
    """

    policies_loaded: bool
    status: HealthResponseStatus
    version: str
    details: Details | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        policies_loaded = self.policies_loaded

        status = self.status.value

        version = self.version

        details: dict[str, Any] | Unset = UNSET
        if not isinstance(self.details, Unset):
            details = self.details.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "policies_loaded": policies_loaded,
                "status": status,
                "version": version,
            }
        )
        if details is not UNSET:
            field_dict["details"] = details

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.details import Details

        d = dict(src_dict)
        policies_loaded = d.pop("policies_loaded")

        status = HealthResponseStatus(d.pop("status"))

        version = d.pop("version")

        _details = d.pop("details", UNSET)
        details: Details | Unset
        if isinstance(_details, Unset):
            details = UNSET
        else:
            details = Details.from_dict(_details)

        health_response = cls(
            policies_loaded=policies_loaded,
            status=status,
            version=version,
            details=details,
        )

        health_response.additional_properties = d
        return health_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MetadataIn")


@_attrs_define
class MetadataIn:
    """Registry metadata.

    Attributes:
        created (None | str | Unset):
        description (None | str | Unset):
        updated (None | str | Unset):
        updated_by (None | str | Unset):
    """

    created: None | str | Unset = UNSET
    description: None | str | Unset = UNSET
    updated: None | str | Unset = UNSET
    updated_by: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created: None | str | Unset
        if isinstance(self.created, Unset):
            created = UNSET
        else:
            created = self.created

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        updated: None | str | Unset
        if isinstance(self.updated, Unset):
            updated = UNSET
        else:
            updated = self.updated

        updated_by: None | str | Unset
        if isinstance(self.updated_by, Unset):
            updated_by = UNSET
        else:
            updated_by = self.updated_by

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created is not UNSET:
            field_dict["created"] = created
        if description is not UNSET:
            field_dict["description"] = description
        if updated is not UNSET:
            field_dict["updated"] = updated
        if updated_by is not UNSET:
            field_dict["updated_by"] = updated_by

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_created(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        created = _parse_created(d.pop("created", UNSET))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_updated(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        updated = _parse_updated(d.pop("updated", UNSET))

        def _parse_updated_by(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        updated_by = _parse_updated_by(d.pop("updated_by", UNSET))

        metadata_in = cls(
            created=created,
            description=description,
            updated=updated,
            updated_by=updated_by,
        )

        metadata_in.additional_properties = d
        return metadata_in

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="LegalInfoIn")


@_attrs_define
class LegalInfoIn:
    """Legal and administrative details.

    Attributes:
        fiscal_code (None | str | Unset):
        legal_form (None | str | Unset):
        name (None | str | Unset):
        registered_office (None | str | Unset):
        registration_number (None | str | Unset):
        vat (None | str | Unset):
    """

    fiscal_code: None | str | Unset = UNSET
    legal_form: None | str | Unset = UNSET
    name: None | str | Unset = UNSET
    registered_office: None | str | Unset = UNSET
    registration_number: None | str | Unset = UNSET
    vat: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        fiscal_code: None | str | Unset
        if isinstance(self.fiscal_code, Unset):
            fiscal_code = UNSET
        else:
            fiscal_code = self.fiscal_code

        legal_form: None | str | Unset
        if isinstance(self.legal_form, Unset):
            legal_form = UNSET
        else:
            legal_form = self.legal_form

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        registered_office: None | str | Unset
        if isinstance(self.registered_office, Unset):
            registered_office = UNSET
        else:
            registered_office = self.registered_office

        registration_number: None | str | Unset
        if isinstance(self.registration_number, Unset):
            registration_number = UNSET
        else:
            registration_number = self.registration_number

        vat: None | str | Unset
        if isinstance(self.vat, Unset):
            vat = UNSET
        else:
            vat = self.vat

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if fiscal_code is not UNSET:
            field_dict["fiscal_code"] = fiscal_code
        if legal_form is not UNSET:
            field_dict["legal_form"] = legal_form
        if name is not UNSET:
            field_dict["name"] = name
        if registered_office is not UNSET:
            field_dict["registered_office"] = registered_office
        if registration_number is not UNSET:
            field_dict["registration_number"] = registration_number
        if vat is not UNSET:
            field_dict["vat"] = vat

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_fiscal_code(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        fiscal_code = _parse_fiscal_code(d.pop("fiscal_code", UNSET))

        def _parse_legal_form(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        legal_form = _parse_legal_form(d.pop("legal_form", UNSET))

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_registered_office(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        registered_office = _parse_registered_office(d.pop("registered_office", UNSET))

        def _parse_registration_number(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        registration_number = _parse_registration_number(d.pop("registration_number", UNSET))

        def _parse_vat(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        vat = _parse_vat(d.pop("vat", UNSET))

        legal_info_in = cls(
            fiscal_code=fiscal_code,
            legal_form=legal_form,
            name=name,
            registered_office=registered_office,
            registration_number=registration_number,
            vat=vat,
        )

        legal_info_in.additional_properties = d
        return legal_info_in

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from dataclasses import dataclass
from typing import Any

import httpx

from celine.sdk.auth.provider import TokenProvider


@dataclass
class AsyncServiceClient:
    """A small helper for curated SDK wrappers.

    This class is *not* tied to generated code; wrappers can use it to build
    an httpx.AsyncClient with consistent auth/headers.
    """

    base_url: str
    token_provider: TokenProvider | None = None
    timeout: float = 10.0

    def _build_client(self, headers: dict[str, str] | None = None) -> httpx.AsyncClient:
        return httpx.AsyncClient(base_url=self.base_url, timeout=self.timeout, headers=headers)

    async def _auth_headers(self) -> dict[str, str]:
        if not self.token_provider:
            return {}
        token = await self.token_provider.get_token()
        return {"Authorization": f"Bearer {token.access_token}"}



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.delivery_point_schema import DeliveryPointSchema
    from ..models.user_member_detail_schema_extra_type_0 import UserMemberDetailSchemaExtraType0


T = TypeVar("T", bound="UserMemberDetailSchema")


@_attrs_define
class UserMemberDetailSchema:
    """
    Attributes:
        area (str):
        key (str):
        name (str):
        role (str):
        status (str):
        created_at (None | str | Unset):
        delivery_points (list[DeliveryPointSchema] | None | Unset):
        extra (None | Unset | UserMemberDetailSchemaExtraType0):
        updated_at (None | str | Unset):
    """

    area: str
    key: str
    name: str
    role: str
    status: str
    created_at: None | str | Unset = UNSET
    delivery_points: list[DeliveryPointSchema] | None | Unset = UNSET
    extra: None | Unset | UserMemberDetailSchemaExtraType0 = UNSET
    updated_at: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.user_member_detail_schema_extra_type_0 import UserMemberDetailSchemaExtraType0

        area = self.area

        key = self.key

        name = self.name

        role = self.role

        status = self.status

        created_at: None | str | Unset
        if isinstance(self.created_at, Unset):
            created_at = UNSET
        else:
            created_at = self.created_at

        delivery_points: list[dict[str, Any]] | None | Unset
        if isinstance(self.delivery_points, Unset):
            delivery_points = UNSET
        elif isinstance(self.delivery_points, list):
            delivery_points = []
            for delivery_points_type_0_item_data in self.delivery_points:
                delivery_points_type_0_item = delivery_points_type_0_item_data.to_dict()
                delivery_points.append(delivery_points_type_0_item)

        else:
            delivery_points = self.delivery_points

        extra: dict[str, Any] | None | Unset
        if isinstance(self.extra, Unset):
            extra = UNSET
        elif isinstance(self.extra, UserMemberDetailSchemaExtraType0):
            extra = self.extra.to_dict()
        else:
            extra = self.extra

        updated_at: None | str | Unset
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = self.updated_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "area": area,
                "key": key,
                "name": name,
                "role": role,
                "status": status,
            }
        )
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if delivery_points is not UNSET:
            field_dict["delivery_points"] = delivery_points
        if extra is not UNSET:
            field_dict["extra"] = extra
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.delivery_point_schema import DeliveryPointSchema
        from ..models.user_member_detail_schema_extra_type_0 import UserMemberDetailSchemaExtraType0

        d = dict(src_dict)
        area = d.pop("area")

        key = d.pop("key")

        name = d.pop("name")

        role = d.pop("role")

        status = d.pop("status")

        def _parse_created_at(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        created_at = _parse_created_at(d.pop("created_at", UNSET))

        def _parse_delivery_points(data: object) -> list[DeliveryPointSchema] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                delivery_points_type_0 = []
                _delivery_points_type_0 = data
                for delivery_points_type_0_item_data in _delivery_points_type_0:
                    delivery_points_type_0_item = DeliveryPointSchema.from_dict(delivery_points_type_0_item_data)

                    delivery_points_type_0.append(delivery_points_type_0_item)

                return delivery_points_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[DeliveryPointSchema] | None | Unset, data)

        delivery_points = _parse_delivery_points(d.pop("delivery_points", UNSET))

        def _parse_extra(data: object) -> None | Unset | UserMemberDetailSchemaExtraType0:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                extra_type_0 = UserMemberDetailSchemaExtraType0.from_dict(data)

                return extra_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserMemberDetailSchemaExtraType0, data)

        extra = _parse_extra(d.pop("extra", UNSET))

        def _parse_updated_at(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        updated_at = _parse_updated_at(d.pop("updated_at", UNSET))

        user_member_detail_schema = cls(
            area=area,
            key=key,
            name=name,
            role=role,
            status=status,
            created_at=created_at,
            delivery_points=delivery_points,
            extra=extra,
            updated_at=updated_at,
        )

        user_member_detail_schema.additional_properties = d
        return user_member_detail_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.asset_relationships_in import AssetRelationshipsIn
    from ..models.device_in import DeviceIn


T = TypeVar("T", bound="HeatPumpAssetIn")


@_attrs_define
class HeatPumpAssetIn:
    """Heat pump asset.

    Attributes:
        name (str):
        thermal_power (float):
        cop (float | None | Unset):
        device (DeviceIn | None | Unset):
        eer (float | None | Unset):
        electrical_power (float | None | Unset):
        heat_pump_type (None | str | Unset):
        installation_date (None | str | Unset):
        refrigerant (None | str | Unset):
        relationships (AssetRelationshipsIn | Unset): Relationships between assets.
        reversible (bool | None | Unset):
        scop (float | None | Unset):
        seer (float | None | Unset):
    """

    name: str
    thermal_power: float
    cop: float | None | Unset = UNSET
    device: DeviceIn | None | Unset = UNSET
    eer: float | None | Unset = UNSET
    electrical_power: float | None | Unset = UNSET
    heat_pump_type: None | str | Unset = UNSET
    installation_date: None | str | Unset = UNSET
    refrigerant: None | str | Unset = UNSET
    relationships: AssetRelationshipsIn | Unset = UNSET
    reversible: bool | None | Unset = UNSET
    scop: float | None | Unset = UNSET
    seer: float | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.device_in import DeviceIn

        name = self.name

        thermal_power = self.thermal_power

        cop: float | None | Unset
        if isinstance(self.cop, Unset):
            cop = UNSET
        else:
            cop = self.cop

        device: dict[str, Any] | None | Unset
        if isinstance(self.device, Unset):
            device = UNSET
        elif isinstance(self.device, DeviceIn):
            device = self.device.to_dict()
        else:
            device = self.device

        eer: float | None | Unset
        if isinstance(self.eer, Unset):
            eer = UNSET
        else:
            eer = self.eer

        electrical_power: float | None | Unset
        if isinstance(self.electrical_power, Unset):
            electrical_power = UNSET
        else:
            electrical_power = self.electrical_power

        heat_pump_type: None | str | Unset
        if isinstance(self.heat_pump_type, Unset):
            heat_pump_type = UNSET
        else:
            heat_pump_type = self.heat_pump_type

        installation_date: None | str | Unset
        if isinstance(self.installation_date, Unset):
            installation_date = UNSET
        else:
            installation_date = self.installation_date

        refrigerant: None | str | Unset
        if isinstance(self.refrigerant, Unset):
            refrigerant = UNSET
        else:
            refrigerant = self.refrigerant

        relationships: dict[str, Any] | Unset = UNSET
        if not isinstance(self.relationships, Unset):
            relationships = self.relationships.to_dict()

        reversible: bool | None | Unset
        if isinstance(self.reversible, Unset):
            reversible = UNSET
        else:
            reversible = self.reversible

        scop: float | None | Unset
        if isinstance(self.scop, Unset):
            scop = UNSET
        else:
            scop = self.scop

        seer: float | None | Unset
        if isinstance(self.seer, Unset):
            seer = UNSET
        else:
            seer = self.seer

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "thermal_power": thermal_power,
            }
        )
        if cop is not UNSET:
            field_dict["cop"] = cop
        if device is not UNSET:
            field_dict["device"] = device
        if eer is not UNSET:
            field_dict["eer"] = eer
        if electrical_power is not UNSET:
            field_dict["electrical_power"] = electrical_power
        if heat_pump_type is not UNSET:
            field_dict["heat_pump_type"] = heat_pump_type
        if installation_date is not UNSET:
            field_dict["installation_date"] = installation_date
        if refrigerant is not UNSET:
            field_dict["refrigerant"] = refrigerant
        if relationships is not UNSET:
            field_dict["relationships"] = relationships
        if reversible is not UNSET:
            field_dict["reversible"] = reversible
        if scop is not UNSET:
            field_dict["scop"] = scop
        if seer is not UNSET:
            field_dict["seer"] = seer

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.asset_relationships_in import AssetRelationshipsIn
        from ..models.device_in import DeviceIn

        d = dict(src_dict)
        name = d.pop("name")

        thermal_power = d.pop("thermal_power")

        def _parse_cop(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        cop = _parse_cop(d.pop("cop", UNSET))

        def _parse_device(data: object) -> DeviceIn | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                device_type_0 = DeviceIn.from_dict(data)

                return device_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DeviceIn | None | Unset, data)

        device = _parse_device(d.pop("device", UNSET))

        def _parse_eer(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        eer = _parse_eer(d.pop("eer", UNSET))

        def _parse_electrical_power(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        electrical_power = _parse_electrical_power(d.pop("electrical_power", UNSET))

        def _parse_heat_pump_type(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        heat_pump_type = _parse_heat_pump_type(d.pop("heat_pump_type", UNSET))

        def _parse_installation_date(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        installation_date = _parse_installation_date(d.pop("installation_date", UNSET))

        def _parse_refrigerant(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        refrigerant = _parse_refrigerant(d.pop("refrigerant", UNSET))

        _relationships = d.pop("relationships", UNSET)
        relationships: AssetRelationshipsIn | Unset
        if isinstance(_relationships, Unset):
            relationships = UNSET
        else:
            relationships = AssetRelationshipsIn.from_dict(_relationships)

        def _parse_reversible(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        reversible = _parse_reversible(d.pop("reversible", UNSET))

        def _parse_scop(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        scop = _parse_scop(d.pop("scop", UNSET))

        def _parse_seer(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        seer = _parse_seer(d.pop("seer", UNSET))

        heat_pump_asset_in = cls(
            name=name,
            thermal_power=thermal_power,
            cop=cop,
            device=device,
            eer=eer,
            electrical_power=electrical_power,
            heat_pump_type=heat_pump_type,
            installation_date=installation_date,
            refrigerant=refrigerant,
            relationships=relationships,
            reversible=reversible,
            scop=scop,
            seer=seer,
        )

        heat_pump_asset_in.additional_properties = d
        return heat_pump_asset_in

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="SendTestResponse")


@_attrs_define
class SendTestResponse:
    """
    Attributes:
        status (str):
        failed (int | None | Unset):
        sent (int | None | Unset):
    """

    status: str
    failed: int | None | Unset = UNSET
    sent: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status = self.status

        failed: int | None | Unset
        if isinstance(self.failed, Unset):
            failed = UNSET
        else:
            failed = self.failed

        sent: int | None | Unset
        if isinstance(self.sent, Unset):
            sent = UNSET
        else:
            sent = self.sent

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
            }
        )
        if failed is not UNSET:
            field_dict["failed"] = failed
        if sent is not UNSET:
            field_dict["sent"] = sent

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        status = d.pop("status")

        def _parse_failed(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        failed = _parse_failed(d.pop("failed", UNSET))

        def _parse_sent(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        sent = _parse_sent(d.pop("sent", UNSET))

        send_test_response = cls(
            status=status,
            failed=failed,
            sent=sent,
        )

        send_test_response.additional_properties = d
        return send_test_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from celine.sdk.settings.models import SdkSettings, OidcSettings, MqttSettings
from celine.sdk.settings.loader import load_settings

__all__ = ["SdkSettings", "OidcSettings", "MqttSettings", "load_settings"]



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.delivery_job_out import DeliveryJobOut


T = TypeVar("T", bound="NudgeCreatedItem")


@_attrs_define
class NudgeCreatedItem:
    """
    Attributes:
        deliveries (list[DeliveryJobOut]):
        nudge_id (str):
        rule_id (str):
    """

    deliveries: list[DeliveryJobOut]
    nudge_id: str
    rule_id: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        deliveries = []
        for deliveries_item_data in self.deliveries:
            deliveries_item = deliveries_item_data.to_dict()
            deliveries.append(deliveries_item)

        nudge_id = self.nudge_id

        rule_id = self.rule_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "deliveries": deliveries,
                "nudge_id": nudge_id,
                "rule_id": rule_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.delivery_job_out import DeliveryJobOut

        d = dict(src_dict)
        deliveries = []
        _deliveries = d.pop("deliveries")
        for deliveries_item_data in _deliveries:
            deliveries_item = DeliveryJobOut.from_dict(deliveries_item_data)

            deliveries.append(deliveries_item)

        nudge_id = d.pop("nudge_id")

        rule_id = d.pop("rule_id")

        nudge_created_item = cls(
            deliveries=deliveries,
            nudge_id=nudge_id,
            rule_id=rule_id,
        )

        nudge_created_item.additional_properties = d
        return nudge_created_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# celine/sdk/auth/static.py
"""
Static token provider — wraps a pre-existing access token.

Used by BFF services that forward the caller's JWT to downstream APIs.
No refresh, no client-credentials flow — just holds the token as-is.
"""
from __future__ import annotations

import time

from celine.sdk.auth.models import AccessToken
from celine.sdk.auth.provider import TokenProvider


class StaticTokenProvider(TokenProvider):
    """Token provider that returns a fixed access token.

    Usage::

        provider = StaticTokenProvider("eyJ...")
        token = await provider.get_token()
        assert token.access_token == "eyJ..."

    The token is assumed valid for the lifetime of the provider instance.
    ``is_valid()`` always returns True (the upstream has already validated it).
    """

    def __init__(self, access_token: str) -> None:
        # Strip "Bearer " prefix if present
        if access_token.lower().startswith("bearer "):
            token = access_token.strip()
            parts = token.split()
            access_token = parts[-1] if parts else token

        self._token = AccessToken(
            access_token=access_token,
            # Far future — we don't manage expiry for forwarded tokens
            expires_at=time.time() + 86400,
        )

    async def get_token(self) -> AccessToken:
        return self._token



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.backend_config import BackendConfig
    from ..models.lineage import Lineage
    from ..models.tags import Tags


T = TypeVar("T", bound="DatasetEntryModel")


@_attrs_define
class DatasetEntryModel:
    """
    Attributes:
        backend_type (str):
        dataset_id (str):
        title (str):
        access_level (None | str | Unset):
        backend_config (BackendConfig | Unset): Backend configuration block for dataset storage.
        description (None | str | Unset):
        expose (bool | Unset):  Default: False.
        landing_page (None | str | Unset):
        language_uris (list[str] | None | Unset):
        license_uri (None | str | Unset):
        lineage (Lineage | None | Unset):
        ontology_path (None | str | Unset):
        publisher_uri (None | str | Unset):
        rights_holder_uri (None | str | Unset):
        schema_override_path (None | str | Unset):
        spatial_uris (list[str] | None | Unset):
        tags (None | Tags | Unset):
    """

    backend_type: str
    dataset_id: str
    title: str
    access_level: None | str | Unset = UNSET
    backend_config: BackendConfig | Unset = UNSET
    description: None | str | Unset = UNSET
    expose: bool | Unset = False
    landing_page: None | str | Unset = UNSET
    language_uris: list[str] | None | Unset = UNSET
    license_uri: None | str | Unset = UNSET
    lineage: Lineage | None | Unset = UNSET
    ontology_path: None | str | Unset = UNSET
    publisher_uri: None | str | Unset = UNSET
    rights_holder_uri: None | str | Unset = UNSET
    schema_override_path: None | str | Unset = UNSET
    spatial_uris: list[str] | None | Unset = UNSET
    tags: None | Tags | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.lineage import Lineage
        from ..models.tags import Tags

        backend_type = self.backend_type

        dataset_id = self.dataset_id

        title = self.title

        access_level: None | str | Unset
        if isinstance(self.access_level, Unset):
            access_level = UNSET
        else:
            access_level = self.access_level

        backend_config: dict[str, Any] | Unset = UNSET
        if not isinstance(self.backend_config, Unset):
            backend_config = self.backend_config.to_dict()

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        expose = self.expose

        landing_page: None | str | Unset
        if isinstance(self.landing_page, Unset):
            landing_page = UNSET
        else:
            landing_page = self.landing_page

        language_uris: list[str] | None | Unset
        if isinstance(self.language_uris, Unset):
            language_uris = UNSET
        elif isinstance(self.language_uris, list):
            language_uris = self.language_uris

        else:
            language_uris = self.language_uris

        license_uri: None | str | Unset
        if isinstance(self.license_uri, Unset):
            license_uri = UNSET
        else:
            license_uri = self.license_uri

        lineage: dict[str, Any] | None | Unset
        if isinstance(self.lineage, Unset):
            lineage = UNSET
        elif isinstance(self.lineage, Lineage):
            lineage = self.lineage.to_dict()
        else:
            lineage = self.lineage

        ontology_path: None | str | Unset
        if isinstance(self.ontology_path, Unset):
            ontology_path = UNSET
        else:
            ontology_path = self.ontology_path

        publisher_uri: None | str | Unset
        if isinstance(self.publisher_uri, Unset):
            publisher_uri = UNSET
        else:
            publisher_uri = self.publisher_uri

        rights_holder_uri: None | str | Unset
        if isinstance(self.rights_holder_uri, Unset):
            rights_holder_uri = UNSET
        else:
            rights_holder_uri = self.rights_holder_uri

        schema_override_path: None | str | Unset
        if isinstance(self.schema_override_path, Unset):
            schema_override_path = UNSET
        else:
            schema_override_path = self.schema_override_path

        spatial_uris: list[str] | None | Unset
        if isinstance(self.spatial_uris, Unset):
            spatial_uris = UNSET
        elif isinstance(self.spatial_uris, list):
            spatial_uris = self.spatial_uris

        else:
            spatial_uris = self.spatial_uris

        tags: dict[str, Any] | None | Unset
        if isinstance(self.tags, Unset):
            tags = UNSET
        elif isinstance(self.tags, Tags):
            tags = self.tags.to_dict()
        else:
            tags = self.tags

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "backend_type": backend_type,
                "dataset_id": dataset_id,
                "title": title,
            }
        )
        if access_level is not UNSET:
            field_dict["access_level"] = access_level
        if backend_config is not UNSET:
            field_dict["backend_config"] = backend_config
        if description is not UNSET:
            field_dict["description"] = description
        if expose is not UNSET:
            field_dict["expose"] = expose
        if landing_page is not UNSET:
            field_dict["landing_page"] = landing_page
        if language_uris is not UNSET:
            field_dict["language_uris"] = language_uris
        if license_uri is not UNSET:
            field_dict["license_uri"] = license_uri
        if lineage is not UNSET:
            field_dict["lineage"] = lineage
        if ontology_path is not UNSET:
            field_dict["ontology_path"] = ontology_path
        if publisher_uri is not UNSET:
            field_dict["publisher_uri"] = publisher_uri
        if rights_holder_uri is not UNSET:
            field_dict["rights_holder_uri"] = rights_holder_uri
        if schema_override_path is not UNSET:
            field_dict["schema_override_path"] = schema_override_path
        if spatial_uris is not UNSET:
            field_dict["spatial_uris"] = spatial_uris
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.backend_config import BackendConfig
        from ..models.lineage import Lineage
        from ..models.tags import Tags

        d = dict(src_dict)
        backend_type = d.pop("backend_type")

        dataset_id = d.pop("dataset_id")

        title = d.pop("title")

        def _parse_access_level(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        access_level = _parse_access_level(d.pop("access_level", UNSET))

        _backend_config = d.pop("backend_config", UNSET)
        backend_config: BackendConfig | Unset
        if isinstance(_backend_config, Unset):
            backend_config = UNSET
        else:
            backend_config = BackendConfig.from_dict(_backend_config)

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        expose = d.pop("expose", UNSET)

        def _parse_landing_page(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        landing_page = _parse_landing_page(d.pop("landing_page", UNSET))

        def _parse_language_uris(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                language_uris_type_0 = cast(list[str], data)

                return language_uris_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        language_uris = _parse_language_uris(d.pop("language_uris", UNSET))

        def _parse_license_uri(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        license_uri = _parse_license_uri(d.pop("license_uri", UNSET))

        def _parse_lineage(data: object) -> Lineage | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                lineage_type_0 = Lineage.from_dict(data)

                return lineage_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(Lineage | None | Unset, data)

        lineage = _parse_lineage(d.pop("lineage", UNSET))

        def _parse_ontology_path(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        ontology_path = _parse_ontology_path(d.pop("ontology_path", UNSET))

        def _parse_publisher_uri(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        publisher_uri = _parse_publisher_uri(d.pop("publisher_uri", UNSET))

        def _parse_rights_holder_uri(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        rights_holder_uri = _parse_rights_holder_uri(d.pop("rights_holder_uri", UNSET))

        def _parse_schema_override_path(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        schema_override_path = _parse_schema_override_path(d.pop("schema_override_path", UNSET))

        def _parse_spatial_uris(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                spatial_uris_type_0 = cast(list[str], data)

                return spatial_uris_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        spatial_uris = _parse_spatial_uris(d.pop("spatial_uris", UNSET))

        def _parse_tags(data: object) -> None | Tags | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                tags_type_0 = Tags.from_dict(data)

                return tags_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Tags | Unset, data)

        tags = _parse_tags(d.pop("tags", UNSET))

        dataset_entry_model = cls(
            backend_type=backend_type,
            dataset_id=dataset_id,
            title=title,
            access_level=access_level,
            backend_config=backend_config,
            description=description,
            expose=expose,
            landing_page=landing_page,
            language_uris=language_uris,
            license_uri=license_uri,
            lineage=lineage,
            ontology_path=ontology_path,
            publisher_uri=publisher_uri,
            rights_holder_uri=rights_holder_uri,
            schema_override_path=schema_override_path,
            spatial_uris=spatial_uris,
            tags=tags,
        )

        dataset_entry_model.additional_properties = d
        return dataset_entry_model

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dataset_query_result_items_item import DatasetQueryResultItemsItem


T = TypeVar("T", bound="DatasetQueryResult")


@_attrs_define
class DatasetQueryResult:
    """
    Attributes:
        count (int):
        items (list[DatasetQueryResultItemsItem]):
        limit (int):
        offset (int):
        total (int | None | Unset):
    """

    count: int
    items: list[DatasetQueryResultItemsItem]
    limit: int
    offset: int
    total: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        count = self.count

        items = []
        for items_item_data in self.items:
            items_item = items_item_data.to_dict()
            items.append(items_item)

        limit = self.limit

        offset = self.offset

        total: int | None | Unset
        if isinstance(self.total, Unset):
            total = UNSET
        else:
            total = self.total

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "count": count,
                "items": items,
                "limit": limit,
                "offset": offset,
            }
        )
        if total is not UNSET:
            field_dict["total"] = total

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.dataset_query_result_items_item import DatasetQueryResultItemsItem

        d = dict(src_dict)
        count = d.pop("count")

        items = []
        _items = d.pop("items")
        for items_item_data in _items:
            items_item = DatasetQueryResultItemsItem.from_dict(items_item_data)

            items.append(items_item)

        limit = d.pop("limit")

        offset = d.pop("offset")

        def _parse_total(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        total = _parse_total(d.pop("total", UNSET))

        dataset_query_result = cls(
            count=count,
            items=items,
            limit=limit,
            offset=offset,
            total=total,
        )

        dataset_query_result.additional_properties = d
        return dataset_query_result

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# generated by datamodel-codegen:
#   filename:  openapi.json
#   timestamp: 2026-02-22T16:52:47+00:00

from __future__ import annotations
from pydantic import BaseModel, Field
from datetime import datetime
from typing import Any


class AdminNotificationOutSchema(BaseModel):
    """
    Admin view – same fields, explicit class for OpenAPI separation.
    """
    body: str = Field(..., title='Body')
    created_at: datetime = Field(..., title='Created At')
    deleted_at: datetime | None = Field(None, title='Deleted At')
    family: str = Field(..., title='Family')
    id: str = Field(..., title='Id')
    nudge_log_id: str | None = Field(None, title='Nudge Log Id')
    read_at: datetime | None = Field(None, title='Read At')
    rule_id: str = Field(..., title='Rule Id')
    severity: str = Field(..., title='Severity')
    status: str = Field(..., title='Status')
    title: str = Field(..., title='Title')
    type: str = Field(..., title='Type')
    user_id: str = Field(..., title='User Id')


class DeliveryJobOutSchema(BaseModel):
    body: str = Field(..., title='Body')
    channel: str = Field(..., title='Channel')
    created_at: datetime = Field(..., title='Created At')
    dedup_key: str = Field(..., title='Dedup Key')
    destination: str = Field(..., title='Destination')
    job_id: str = Field(..., title='Job Id')
    nudge_id: str = Field(..., title='Nudge Id')
    rule_id: str = Field(..., title='Rule Id')
    title: str = Field(..., title='Title')
    user_id: str = Field(..., title='User Id')


class DigitalTwinEventSchema(BaseModel):
    event_type: str = Field(..., title='Event Type')
    facts: dict[str, Any] | None = Field(None, description='Enriched facts computed by Digital Twin', title='Facts')
    payload: dict[str, Any] | None = Field(None, title='Payload')
    timestamp: datetime | None = Field(None, title='Timestamp')
    user_id: str = Field(..., title='User Id')


class EngineResultOutSchema(BaseModel):
    details: dict[str, Any] | None = Field(None, title='Details')
    reason: str | None = Field(None, title='Reason')
    status: str = Field(..., title='Status')


class IngestErrorDetailSchema(BaseModel):
    error: str = Field(..., title='Error')
    errors: list[str] | None = Field(None, title='Errors')
    reason: str | None = Field(None, title='Reason')
    results: list[EngineResultOutSchema] | None = Field(None, title='Results')


class NotificationOutSchema(BaseModel):
    """
    User-facing notification. Fields are proper columns, not buried JSON.
    """
    body: str = Field(..., title='Body')
    created_at: datetime = Field(..., title='Created At')
    deleted_at: datetime | None = Field(None, description='Null if not soft-deleted', title='Deleted At')
    family: str = Field(..., description='energy | onboarding | seasonal | …', title='Family')
    id: str = Field(..., title='Id')
    nudge_log_id: str | None = Field(None, description='Originating engine audit row', title='Nudge Log Id')
    read_at: datetime | None = Field(None, description='Null if unread', title='Read At')
    rule_id: str = Field(..., title='Rule Id')
    severity: str = Field(..., description='info | warning | critical', title='Severity')
    status: str = Field(..., description='pending | sent | suppressed | failed', title='Status')
    title: str = Field(..., title='Title')
    type: str = Field(..., description='informative | opportunity | alert', title='Type')
    user_id: str = Field(..., title='User Id')


class NudgeCreatedItemSchema(BaseModel):
    deliveries: list[DeliveryJobOutSchema] = Field(..., title='Deliveries')
    nudge_id: str = Field(..., title='Nudge Id')
    rule_id: str = Field(..., title='Rule Id')


class SendTestRequestSchema(BaseModel):
    """
    Admin-only: user_id is explicit because an admin targets any user.
    """
    body: str | None = Field('Hello!', examples=['Hello!'], title='Body')
    title: str | None = Field('Test', examples=['Test'], title='Title')
    url: str | None = Field('/', examples=['/'], title='Url')
    user_id: str = Field(..., description='Target user ID', title='User Id')


class SendTestResponseSchema(BaseModel):
    failed: int | None = Field(None, title='Failed')
    sent: int | None = Field(None, title='Sent')
    status: str = Field(..., title='Status')


class StatusResponseSchema(BaseModel):
    status: str = Field(..., examples=['ok'], title='Status')


class UnsubscribeRequestSchema(BaseModel):
    """
    user_id is derived from the JWT – not accepted from the caller.
    """
    endpoint: str = Field(..., title='Endpoint')


class ValidationErrorSchema(BaseModel):
    ctx: dict[str, Any] | None = Field(None, title='Context')
    input: Any | None = Field(None, title='Input')
    loc: list[str | int] = Field(..., title='Location')
    msg: str = Field(..., title='Message')
    type: str = Field(..., title='Error Type')


class VapidPublicKeyResponseSchema(BaseModel):
    public_key: str = Field(..., title='Public Key')


class WebPushKeysInSchema(BaseModel):
    auth: str = Field(..., title='Auth')
    p256dh: str = Field(..., title='P256Dh')


class WebPushSubscriptionInSchema(BaseModel):
    endpoint: str = Field(..., title='Endpoint')
    keys: WebPushKeysInSchema


class HTTPValidationErrorSchema(BaseModel):
    detail: list[ValidationErrorSchema] | None = Field(None, title='Detail')


class IngestAcceptedResponseSchema(BaseModel):
    created: list[NudgeCreatedItemSchema] = Field(..., title='Created')
    delivery: str | None = Field('suppressed', examples=['suppressed'], title='Delivery')
    status: str | None = Field('accepted', examples=['accepted'], title='Status')
    suppressed: list[EngineResultOutSchema] = Field(..., title='Suppressed')


class IngestOkResponseSchema(BaseModel):
    created: list[NudgeCreatedItemSchema] = Field(..., title='Created')
    status: str | None = Field('ok', examples=['ok'], title='Status')
    suppressed: list[EngineResultOutSchema] = Field(..., title='Suppressed')


class SubscribeRequestSchema(BaseModel):
    """
    user_id is derived from the JWT – not accepted from the caller.
    """
    subscription: WebPushSubscriptionInSchema



# generated by datamodel-codegen:
#   filename:  openapi.json
#   timestamp: 2026-02-22T16:52:43+00:00

from __future__ import annotations
from pydantic import BaseModel, Extra, Field
from typing import Any


class BackendConfigSchema(BaseModel):
    """
    Backend configuration block for dataset storage.
    """
    format: str | None = Field(None, title='Format')
    path: str | None = Field(None, title='Path')
    public_url: str | None = Field(None, title='Public Url')
    size_bytes: int | None = Field(None, title='Size Bytes')
    table: str | None = Field(None, title='Table')


class CatalogueImportResponseSchema(BaseModel):
    created: int = Field(..., title='Created')
    updated: int = Field(..., title='Updated')


class ContactPointSchema(BaseModel):
    email: str | None = Field(None, title='Email')
    fn: str | None = Field(None, title='Fn')


class DatasetQueryModelSchema(BaseModel):
    limit: int | None = Field(100, title='Limit')
    offset: int | None = Field(0, title='Offset')
    sql: str | None = Field(None, title='Sql')


class DatasetQueryResultSchema(BaseModel):
    count: int = Field(..., title='Count')
    items: list[dict[str, Any]] = Field(..., title='Items')
    limit: int = Field(..., title='Limit')
    offset: int = Field(..., title='Offset')
    total: int | None = Field(None, title='Total')


class LineageSchema(BaseModel):
    class Config:
        extra = Extra.allow
    createdAt: str | None = Field(None, title='Createdat')
    description: str | None = Field(None, title='Description')
    facets: dict[str, Any] | None = Field(None, title='Facets')
    lastLifecycleState: str | None = Field(None, title='Lastlifecyclestate')
    lastModifiedAt: str | None = Field(None, title='Lastmodifiedat')
    name: str | None = Field(None, title='Name')
    namespace: str | None = Field(None, title='Namespace')
    sourceName: str | None = Field(None, title='Sourcename')
    tags: list[str] | None = Field(None, title='Tags')
    title: str | None = Field(None, title='Title')
    updatedAt: str | None = Field(None, title='Updatedat')


class TemporalCoverageSchema(BaseModel):
    end: str | None = Field(None, title='End')
    start: str | None = Field(None, title='Start')


class ValidationErrorSchema(BaseModel):
    loc: list[str | int] = Field(..., title='Location')
    msg: str = Field(..., title='Message')
    type: str = Field(..., title='Error Type')


class HTTPValidationErrorSchema(BaseModel):
    detail: list[ValidationErrorSchema] | None = Field(None, title='Detail')


class TagsSchema(BaseModel):
    accessRights: str | None = Field(None, title='Accessrights')
    accrualPeriodicity: str | None = Field(None, title='Accrualperiodicity')
    conformsTo: str | None = Field(None, title='Conformsto')
    contactPoint: ContactPointSchema | None = None
    identifier: str | None = Field(None, title='Identifier')
    keywords: list[str] | None = Field(None, title='Keywords')
    temporal: TemporalCoverageSchema | None = None
    themes: list[str] | None = Field(None, title='Themes')


class DatasetEntryModelSchema(BaseModel):
    access_level: str | None = Field(None, title='Access Level')
    backend_config: BackendConfigSchema | None = None
    backend_type: str = Field(..., title='Backend Type')
    dataset_id: str = Field(..., title='Dataset Id')
    description: str | None = Field(None, title='Description')
    expose: bool | None = Field(False, title='Expose')
    landing_page: str | None = Field(None, title='Landing Page')
    language_uris: list[str] | None = Field(None, title='Language Uris')
    license_uri: str | None = Field(None, title='License Uri')
    lineage: LineageSchema | None = None
    ontology_path: str | None = Field(None, title='Ontology Path')
    publisher_uri: str | None = Field(None, title='Publisher Uri')
    rights_holder_uri: str | None = Field(None, title='Rights Holder Uri')
    schema_override_path: str | None = Field(None, title='Schema Override Path')
    spatial_uris: list[str] | None = Field(None, title='Spatial Uris')
    tags: TagsSchema | None = None
    title: str = Field(..., title='Title')


class CatalogueImportModelSchema(BaseModel):
    """
    Full catalogue import: list of datasets.
    """
    datasets: list[DatasetEntryModelSchema] = Field(..., title='Datasets')



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_community_summary_schema import UserCommunitySummarySchema
    from ..models.user_member_summary_schema import UserMemberSummarySchema
    from ..models.user_membership_schema_assets_count_type_0 import UserMembershipSchemaAssetsCountType0


T = TypeVar("T", bound="UserMembershipSchema")


@_attrs_define
class UserMembershipSchema:
    """
    Attributes:
        community (UserCommunitySummarySchema):
        member (UserMemberSummarySchema):
        assets_count (None | Unset | UserMembershipSchemaAssetsCountType0):
        delivery_points_count (int | None | Unset):  Default: 0.
    """

    community: UserCommunitySummarySchema
    member: UserMemberSummarySchema
    assets_count: None | Unset | UserMembershipSchemaAssetsCountType0 = UNSET
    delivery_points_count: int | None | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.user_membership_schema_assets_count_type_0 import UserMembershipSchemaAssetsCountType0

        community = self.community.to_dict()

        member = self.member.to_dict()

        assets_count: dict[str, Any] | None | Unset
        if isinstance(self.assets_count, Unset):
            assets_count = UNSET
        elif isinstance(self.assets_count, UserMembershipSchemaAssetsCountType0):
            assets_count = self.assets_count.to_dict()
        else:
            assets_count = self.assets_count

        delivery_points_count: int | None | Unset
        if isinstance(self.delivery_points_count, Unset):
            delivery_points_count = UNSET
        else:
            delivery_points_count = self.delivery_points_count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "community": community,
                "member": member,
            }
        )
        if assets_count is not UNSET:
            field_dict["assets_count"] = assets_count
        if delivery_points_count is not UNSET:
            field_dict["delivery_points_count"] = delivery_points_count

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user_community_summary_schema import UserCommunitySummarySchema
        from ..models.user_member_summary_schema import UserMemberSummarySchema
        from ..models.user_membership_schema_assets_count_type_0 import UserMembershipSchemaAssetsCountType0

        d = dict(src_dict)
        community = UserCommunitySummarySchema.from_dict(d.pop("community"))

        member = UserMemberSummarySchema.from_dict(d.pop("member"))

        def _parse_assets_count(data: object) -> None | Unset | UserMembershipSchemaAssetsCountType0:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                assets_count_type_0 = UserMembershipSchemaAssetsCountType0.from_dict(data)

                return assets_count_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserMembershipSchemaAssetsCountType0, data)

        assets_count = _parse_assets_count(d.pop("assets_count", UNSET))

        def _parse_delivery_points_count(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        delivery_points_count = _parse_delivery_points_count(d.pop("delivery_points_count", UNSET))

        user_membership_schema = cls(
            community=community,
            member=member,
            assets_count=assets_count,
            delivery_points_count=delivery_points_count,
        )

        user_membership_schema.additional_properties = d
        return user_membership_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.asset_relationships_in import AssetRelationshipsIn
    from ..models.device_in import DeviceIn


T = TypeVar("T", bound="MeterAssetIn")


@_attrs_define
class MeterAssetIn:
    """Meter asset.

    Attributes:
        meter_type (str):
        name (str):
        sensor_id (str):
        device (DeviceIn | None | Unset):
        installation_date (None | str | Unset):
        pod (None | str | Unset):
        protocol (None | str | Unset):
        relationships (AssetRelationshipsIn | Unset): Relationships between assets.
    """

    meter_type: str
    name: str
    sensor_id: str
    device: DeviceIn | None | Unset = UNSET
    installation_date: None | str | Unset = UNSET
    pod: None | str | Unset = UNSET
    protocol: None | str | Unset = UNSET
    relationships: AssetRelationshipsIn | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.device_in import DeviceIn

        meter_type = self.meter_type

        name = self.name

        sensor_id = self.sensor_id

        device: dict[str, Any] | None | Unset
        if isinstance(self.device, Unset):
            device = UNSET
        elif isinstance(self.device, DeviceIn):
            device = self.device.to_dict()
        else:
            device = self.device

        installation_date: None | str | Unset
        if isinstance(self.installation_date, Unset):
            installation_date = UNSET
        else:
            installation_date = self.installation_date

        pod: None | str | Unset
        if isinstance(self.pod, Unset):
            pod = UNSET
        else:
            pod = self.pod

        protocol: None | str | Unset
        if isinstance(self.protocol, Unset):
            protocol = UNSET
        else:
            protocol = self.protocol

        relationships: dict[str, Any] | Unset = UNSET
        if not isinstance(self.relationships, Unset):
            relationships = self.relationships.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "meter_type": meter_type,
                "name": name,
                "sensor_id": sensor_id,
            }
        )
        if device is not UNSET:
            field_dict["device"] = device
        if installation_date is not UNSET:
            field_dict["installation_date"] = installation_date
        if pod is not UNSET:
            field_dict["pod"] = pod
        if protocol is not UNSET:
            field_dict["protocol"] = protocol
        if relationships is not UNSET:
            field_dict["relationships"] = relationships

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.asset_relationships_in import AssetRelationshipsIn
        from ..models.device_in import DeviceIn

        d = dict(src_dict)
        meter_type = d.pop("meter_type")

        name = d.pop("name")

        sensor_id = d.pop("sensor_id")

        def _parse_device(data: object) -> DeviceIn | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                device_type_0 = DeviceIn.from_dict(data)

                return device_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DeviceIn | None | Unset, data)

        device = _parse_device(d.pop("device", UNSET))

        def _parse_installation_date(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        installation_date = _parse_installation_date(d.pop("installation_date", UNSET))

        def _parse_pod(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        pod = _parse_pod(d.pop("pod", UNSET))

        def _parse_protocol(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        protocol = _parse_protocol(d.pop("protocol", UNSET))

        _relationships = d.pop("relationships", UNSET)
        relationships: AssetRelationshipsIn | Unset
        if isinstance(_relationships, Unset):
            relationships = UNSET
        else:
            relationships = AssetRelationshipsIn.from_dict(_relationships)

        meter_asset_in = cls(
            meter_type=meter_type,
            name=name,
            sensor_id=sensor_id,
            device=device,
            installation_date=installation_date,
            pod=pod,
            protocol=protocol,
            relationships=relationships,
        )

        meter_asset_in.additional_properties = d
        return meter_asset_in

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.context import Context


T = TypeVar("T", bound="Action")


@_attrs_define
class Action:
    """Represents the action being performed.

    Attributes:
        name (str): Action name (read, write, subscribe, etc.)
        context (Context | Unset): Action-specific context (e.g., state transitions)
    """

    name: str
    context: Context | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        context: dict[str, Any] | Unset = UNSET
        if not isinstance(self.context, Unset):
            context = self.context.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if context is not UNSET:
            field_dict["context"] = context

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.context import Context

        d = dict(src_dict)
        name = d.pop("name")

        _context = d.pop("context", UNSET)
        context: Context | Unset
        if isinstance(_context, Unset):
            context = UNSET
        else:
            context = Context.from_dict(_context)

        action = cls(
            name=name,
            context=context,
        )

        action.additional_properties = d
        return action

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from dataclasses import dataclass

import httpx


@dataclass(frozen=True)
class OidcConfiguration:
    issuer: str
    token_endpoint: str
    jwks_uri: str


class OidcDiscoveryClient:
    def __init__(self, issuer_base_url: str, timeout: float = 10.0):
        self._issuer = issuer_base_url.rstrip("/")
        self._timeout = timeout
        self._config: OidcConfiguration | None = None

    async def get_config(self) -> OidcConfiguration:
        if self._config:
            return self._config

        url = f"{self._issuer}/.well-known/openid-configuration"
        async with httpx.AsyncClient(timeout=self._timeout) as client:
            r = await client.get(url)
            r.raise_for_status()
            payload = r.json()

        self._config = OidcConfiguration(
            issuer=payload["issuer"],
            token_endpoint=payload["token_endpoint"],
            jwks_uri=payload["jwks_uri"],
        )
        return self._config



from __future__ import annotations

import json
from pathlib import Path
from typing import Any

import httpx
import yaml


def parse_openapi_bytes(content: bytes) -> dict[str, Any]:
    # Try JSON first
    try:
        return json.loads(content)
    except Exception:
        pass
    # YAML fallback
    obj = yaml.safe_load(content.decode("utf-8"))
    if not isinstance(obj, dict):
        raise ValueError("OpenAPI spec must be an object")
    return obj


def spec_version(spec: dict[str, Any]) -> str:
    info = spec.get("info") or {}
    v = info.get("version")
    if not isinstance(v, str) or not v:
        raise ValueError("OpenAPI spec missing info.version")
    return v


def fetch_spec(url: str, timeout: float = 20.0) -> dict[str, Any]:
    r = httpx.get(url, timeout=timeout)
    r.raise_for_status()
    return parse_openapi_bytes(r.content)


def write_spec(root: Path, service: str, version: str, spec: dict[str, Any]) -> Path:
    base = root / service
    ver_dir = base / f"v{version}"
    ver_dir.mkdir(parents=True, exist_ok=True)
    path = ver_dir / "openapi.json"
    path.write_text(json.dumps(spec, indent=2, sort_keys=True))

    return path


def list_versions(root: Path, service: str) -> list[str]:
    base = root / service
    if not base.exists():
        return []
    versions = []
    for p in base.iterdir():
        if p.is_dir() and p.name.startswith("v"):
            versions.append(p.name)
    return sorted(versions)


def latest_version(root: Path, service: str) -> str | None:
    vs = list_versions(root, service)
    if not vs:
        return None
    return vs[-1]



"""Policy evaluation module for CELINE SDK.

This module provides in-process policy evaluation using regorus (embedded OPA).

Example usage:
    ```python
    from celine.sdk.policies import (
        PolicyEngine,
        CachedPolicyEngine,
        DecisionCache,
        PolicyInput,
        Subject,
        Resource,
        Action,
        ResourceType,
        SubjectType,
    )

    # Initialize engine
    engine = PolicyEngine(
        policies_dir="./policies",
        data_dir="./policies/data"  # optional
    )
    engine.load()

    # Optionally wrap with cache
    cached_engine = CachedPolicyEngine(
        engine=engine,
        cache=DecisionCache(maxsize=10000, ttl_seconds=300)
    )

    # Evaluate a policy
    policy_input = PolicyInput(
        subject=Subject(
            id="user-123",
            type=SubjectType.USER,
            groups=["engineering"],
            scopes=["dataset.read"]
        ),
        resource=Resource(
            type=ResourceType.DATASET,
            id="dataset-456",
            attributes={"access_level": "internal"}
        ),
        action=Action(name="read"),
        environment={"timestamp": 1234567890}
    )

    decision = cached_engine.evaluate_decision(
        policy_package="celine.dataset.access",
        policy_input=policy_input
    )

    if decision.allowed:
        print(f"Access granted: {decision.reason}")
    else:
        print(f"Access denied: {decision.reason}")
    ```
"""

from celine.sdk.policies.cache import CachedPolicyEngine, DecisionCache
from celine.sdk.policies.engine import PolicyEngine, PolicyEngineError
from celine.sdk.policies.models import (
    Action,
    Decision,
    FilterPredicate,
    PolicyInput,
    Resource,
    ResourceType,
    Subject,
    SubjectType,
)

__all__ = [
    # Engine
    "PolicyEngine",
    "PolicyEngineError",
    "CachedPolicyEngine",
    "DecisionCache",
    # Models
    "Subject",
    "SubjectType",
    "Resource",
    "ResourceType",
    "Action",
    "PolicyInput",
    "Decision",
    "FilterPredicate",
]



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="DeliveryJobOut")


@_attrs_define
class DeliveryJobOut:
    """
    Attributes:
        body (str):
        channel (str):
        created_at (datetime.datetime):
        dedup_key (str):
        destination (str):
        job_id (str):
        nudge_id (str):
        rule_id (str):
        title (str):
        user_id (str):
    """

    body: str
    channel: str
    created_at: datetime.datetime
    dedup_key: str
    destination: str
    job_id: str
    nudge_id: str
    rule_id: str
    title: str
    user_id: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        body = self.body

        channel = self.channel

        created_at = self.created_at.isoformat()

        dedup_key = self.dedup_key

        destination = self.destination

        job_id = self.job_id

        nudge_id = self.nudge_id

        rule_id = self.rule_id

        title = self.title

        user_id = self.user_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "body": body,
                "channel": channel,
                "created_at": created_at,
                "dedup_key": dedup_key,
                "destination": destination,
                "job_id": job_id,
                "nudge_id": nudge_id,
                "rule_id": rule_id,
                "title": title,
                "user_id": user_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        body = d.pop("body")

        channel = d.pop("channel")

        created_at = isoparse(d.pop("created_at"))

        dedup_key = d.pop("dedup_key")

        destination = d.pop("destination")

        job_id = d.pop("job_id")

        nudge_id = d.pop("nudge_id")

        rule_id = d.pop("rule_id")

        title = d.pop("title")

        user_id = d.pop("user_id")

        delivery_job_out = cls(
            body=body,
            channel=channel,
            created_at=created_at,
            dedup_key=dedup_key,
            destination=destination,
            job_id=job_id,
            nudge_id=nudge_id,
            rule_id=rule_id,
            title=title,
            user_id=user_id,
        )

        delivery_job_out.additional_properties = d
        return delivery_job_out

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# celine/sdk/dt/community.py
"""
CommunityClient — curated async wrapper for the it_energy_community domain.

All methods delegate to the generated ``celine.sdk.openapi.dt.api.it_energy_community``
module functions. Error handling converts generated error types into clean exceptions.
"""
from __future__ import annotations

import logging
from typing import Any, TYPE_CHECKING

from celine.sdk.dt.util import unwrap, DTApiError

from celine.sdk.openapi.dt.types import UNSET

from celine.sdk.openapi.dt.models import (
    HTTPValidationError,
    ResponseItEnergyCommunityGetInfo,
    ValuesRequestSchema,
    ValueDescriptorSchema,
    GenericPayload,
    SimulationDescriptorSchema,
    FetchResultSchema,
)
from celine.sdk.openapi.dt.api.it_energy_community import (
    it_energy_community_energy_balance as _energy_balance,
    it_energy_community_energy_balance_hourly as _energy_balance_hourly,
    it_energy_community_get_info as _info,
    it_energy_community_list_values as _list_values,
    it_energy_community_fetch_values_post as _post_value,
    it_energy_community_describe_value as _describe_value,
    it_energy_community_list_simulations as _list_simulations,
)


if TYPE_CHECKING:
    from celine.sdk.dt.client import DTClient

logger = logging.getLogger(__name__)


class CommunityClient:
    """Async client for the Italian Energy Community DT domain.

    Not instantiated directly — use ``DTClient.communities``.
    """

    def __init__(self, dt: DTClient) -> None:
        self._dt = dt

    async def info(self, community_id: str) -> ResponseItEnergyCommunityGetInfo:
        """Get domain info and available capabilities for a community."""
        client = await self._dt._get_client()
        result = await _info.asyncio_detailed(community_id=community_id, client=client)
        data = unwrap(result)
        if isinstance(data, HTTPValidationError):
            logger.warning(data.detail)
            raise DTApiError("Validation error", 500)
        return data

    async def energy_balance(
        self,
        community_id: str,
        *,
        start: str | None = None,
        end: str | None = None,
    ) -> dict[str, Any]:
        """Get the energy balance for a community.

        Args:
            community_id: Community entity ID.
            start: ISO timestamp for period start (optional).
            end: ISO timestamp for period end (optional).

        Returns:
            Energy balance dict with production, consumption, self-consumption.
        """
        client = await self._dt._get_client()
        result = await _energy_balance.asyncio_detailed(
            community_id=community_id,
            client=client,
            start=start if start else UNSET,
            end=end if end else UNSET,
        )
        data = unwrap(result)
        if isinstance(data, HTTPValidationError):
            logger.warning(data.detail)
            raise DTApiError("Validation error", 500)
        return data

    async def list_values(self, community_id: str) -> list[ValueDescriptorSchema]:
        """List available value fetchers for a participant."""
        client = await self._dt._get_client()
        result = await _list_values.asyncio_detailed(
            community_id=community_id, client=client
        )
        data = unwrap(result)
        if isinstance(data, HTTPValidationError):
            logger.warning(data.detail)
            raise DTApiError("Validation error", 500)
        return data

    async def fetch_values(
        self,
        community_id: str,
        fetcher_id: str,
        payload: dict[str, Any] = {},
        limit: int | None = None,
        offset: int | None = None,
    ) -> FetchResultSchema:
        """Fetch a value using a JSON payload (POST)."""
        client = await self._dt._get_client()

        if limit is not None:
            payload["limit"] = limit
        if offset is not None:
            payload["offset"] = offset

        body = ValuesRequestSchema(payload=GenericPayload.from_dict(payload))

        result = await _post_value.asyncio_detailed(
            community_id=community_id,
            fetcher_id=fetcher_id,
            client=client,
            body=body,
        )
        data = unwrap(result)
        if isinstance(data, HTTPValidationError):
            logger.warning(data.detail)
            raise DTApiError("Validation error", 500)
        return data

    async def describe_value(
        self,
        community_id: str,
        fetcher_id: str,
    ) -> ValueDescriptorSchema:
        """Describe a value fetcher's schema and metadata."""
        client = await self._dt._get_client()
        result = await _describe_value.asyncio_detailed(
            community_id=community_id,
            fetcher_id=fetcher_id,
            client=client,
        )
        data = unwrap(result)
        if isinstance(data, HTTPValidationError):
            logger.warning(data.detail)
            raise DTApiError("Validation error", 500)
        return data

    async def list_simulations(
        self, community_id: str
    ) -> list[SimulationDescriptorSchema]:
        """List available simulations for a participant."""
        client = await self._dt._get_client()
        result = await _list_simulations.asyncio_detailed(
            community_id=community_id, client=client
        )
        data = unwrap(result)
        if isinstance(data, HTTPValidationError):
            logger.warning(data.detail)
            raise DTApiError("Validation error", 500)
        return data



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.contact_point import ContactPoint
    from ..models.temporal_coverage import TemporalCoverage


T = TypeVar("T", bound="Tags")


@_attrs_define
class Tags:
    """
    Attributes:
        access_rights (None | str | Unset):
        accrual_periodicity (None | str | Unset):
        conforms_to (None | str | Unset):
        contact_point (ContactPoint | None | Unset):
        identifier (None | str | Unset):
        keywords (list[str] | None | Unset):
        temporal (None | TemporalCoverage | Unset):
        themes (list[str] | None | Unset):
    """

    access_rights: None | str | Unset = UNSET
    accrual_periodicity: None | str | Unset = UNSET
    conforms_to: None | str | Unset = UNSET
    contact_point: ContactPoint | None | Unset = UNSET
    identifier: None | str | Unset = UNSET
    keywords: list[str] | None | Unset = UNSET
    temporal: None | TemporalCoverage | Unset = UNSET
    themes: list[str] | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.contact_point import ContactPoint
        from ..models.temporal_coverage import TemporalCoverage

        access_rights: None | str | Unset
        if isinstance(self.access_rights, Unset):
            access_rights = UNSET
        else:
            access_rights = self.access_rights

        accrual_periodicity: None | str | Unset
        if isinstance(self.accrual_periodicity, Unset):
            accrual_periodicity = UNSET
        else:
            accrual_periodicity = self.accrual_periodicity

        conforms_to: None | str | Unset
        if isinstance(self.conforms_to, Unset):
            conforms_to = UNSET
        else:
            conforms_to = self.conforms_to

        contact_point: dict[str, Any] | None | Unset
        if isinstance(self.contact_point, Unset):
            contact_point = UNSET
        elif isinstance(self.contact_point, ContactPoint):
            contact_point = self.contact_point.to_dict()
        else:
            contact_point = self.contact_point

        identifier: None | str | Unset
        if isinstance(self.identifier, Unset):
            identifier = UNSET
        else:
            identifier = self.identifier

        keywords: list[str] | None | Unset
        if isinstance(self.keywords, Unset):
            keywords = UNSET
        elif isinstance(self.keywords, list):
            keywords = self.keywords

        else:
            keywords = self.keywords

        temporal: dict[str, Any] | None | Unset
        if isinstance(self.temporal, Unset):
            temporal = UNSET
        elif isinstance(self.temporal, TemporalCoverage):
            temporal = self.temporal.to_dict()
        else:
            temporal = self.temporal

        themes: list[str] | None | Unset
        if isinstance(self.themes, Unset):
            themes = UNSET
        elif isinstance(self.themes, list):
            themes = self.themes

        else:
            themes = self.themes

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if access_rights is not UNSET:
            field_dict["accessRights"] = access_rights
        if accrual_periodicity is not UNSET:
            field_dict["accrualPeriodicity"] = accrual_periodicity
        if conforms_to is not UNSET:
            field_dict["conformsTo"] = conforms_to
        if contact_point is not UNSET:
            field_dict["contactPoint"] = contact_point
        if identifier is not UNSET:
            field_dict["identifier"] = identifier
        if keywords is not UNSET:
            field_dict["keywords"] = keywords
        if temporal is not UNSET:
            field_dict["temporal"] = temporal
        if themes is not UNSET:
            field_dict["themes"] = themes

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.contact_point import ContactPoint
        from ..models.temporal_coverage import TemporalCoverage

        d = dict(src_dict)

        def _parse_access_rights(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        access_rights = _parse_access_rights(d.pop("accessRights", UNSET))

        def _parse_accrual_periodicity(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        accrual_periodicity = _parse_accrual_periodicity(d.pop("accrualPeriodicity", UNSET))

        def _parse_conforms_to(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        conforms_to = _parse_conforms_to(d.pop("conformsTo", UNSET))

        def _parse_contact_point(data: object) -> ContactPoint | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                contact_point_type_0 = ContactPoint.from_dict(data)

                return contact_point_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ContactPoint | None | Unset, data)

        contact_point = _parse_contact_point(d.pop("contactPoint", UNSET))

        def _parse_identifier(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        identifier = _parse_identifier(d.pop("identifier", UNSET))

        def _parse_keywords(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                keywords_type_0 = cast(list[str], data)

                return keywords_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        keywords = _parse_keywords(d.pop("keywords", UNSET))

        def _parse_temporal(data: object) -> None | TemporalCoverage | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                temporal_type_0 = TemporalCoverage.from_dict(data)

                return temporal_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | TemporalCoverage | Unset, data)

        temporal = _parse_temporal(d.pop("temporal", UNSET))

        def _parse_themes(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                themes_type_0 = cast(list[str], data)

                return themes_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        themes = _parse_themes(d.pop("themes", UNSET))

        tags = cls(
            access_rights=access_rights,
            accrual_periodicity=accrual_periodicity,
            conforms_to=conforms_to,
            contact_point=contact_point,
            identifier=identifier,
            keywords=keywords,
            temporal=temporal,
            themes=themes,
        )

        tags.additional_properties = d
        return tags

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_community_summary import UserCommunitySummary
    from ..models.user_member_summary import UserMemberSummary
    from ..models.user_membership_assets_count import UserMembershipAssetsCount


T = TypeVar("T", bound="UserMembership")


@_attrs_define
class UserMembership:
    """
    Attributes:
        community (UserCommunitySummary):
        member (UserMemberSummary):
        assets_count (UserMembershipAssetsCount | Unset):
        delivery_points_count (int | Unset):  Default: 0.
    """

    community: UserCommunitySummary
    member: UserMemberSummary
    assets_count: UserMembershipAssetsCount | Unset = UNSET
    delivery_points_count: int | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        community = self.community.to_dict()

        member = self.member.to_dict()

        assets_count: dict[str, Any] | Unset = UNSET
        if not isinstance(self.assets_count, Unset):
            assets_count = self.assets_count.to_dict()

        delivery_points_count = self.delivery_points_count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "community": community,
                "member": member,
            }
        )
        if assets_count is not UNSET:
            field_dict["assets_count"] = assets_count
        if delivery_points_count is not UNSET:
            field_dict["delivery_points_count"] = delivery_points_count

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user_community_summary import UserCommunitySummary
        from ..models.user_member_summary import UserMemberSummary
        from ..models.user_membership_assets_count import UserMembershipAssetsCount

        d = dict(src_dict)
        community = UserCommunitySummary.from_dict(d.pop("community"))

        member = UserMemberSummary.from_dict(d.pop("member"))

        _assets_count = d.pop("assets_count", UNSET)
        assets_count: UserMembershipAssetsCount | Unset
        if isinstance(_assets_count, Unset):
            assets_count = UNSET
        else:
            assets_count = UserMembershipAssetsCount.from_dict(_assets_count)

        delivery_points_count = d.pop("delivery_points_count", UNSET)

        user_membership = cls(
            community=community,
            member=member,
            assets_count=assets_count,
            delivery_points_count=delivery_points_count,
        )

        user_membership.additional_properties = d
        return user_membership

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DeliveryPointSchema")


@_attrs_define
class DeliveryPointSchema:
    """
    Attributes:
        id (str):
        type_ (str):
        active (bool | None | Unset):  Default: True.
        address (None | str | Unset):
        description (None | str | Unset):
        tariff (None | str | Unset):
    """

    id: str
    type_: str
    active: bool | None | Unset = True
    address: None | str | Unset = UNSET
    description: None | str | Unset = UNSET
    tariff: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        type_ = self.type_

        active: bool | None | Unset
        if isinstance(self.active, Unset):
            active = UNSET
        else:
            active = self.active

        address: None | str | Unset
        if isinstance(self.address, Unset):
            address = UNSET
        else:
            address = self.address

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        tariff: None | str | Unset
        if isinstance(self.tariff, Unset):
            tariff = UNSET
        else:
            tariff = self.tariff

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "type": type_,
            }
        )
        if active is not UNSET:
            field_dict["active"] = active
        if address is not UNSET:
            field_dict["address"] = address
        if description is not UNSET:
            field_dict["description"] = description
        if tariff is not UNSET:
            field_dict["tariff"] = tariff

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        type_ = d.pop("type")

        def _parse_active(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        active = _parse_active(d.pop("active", UNSET))

        def _parse_address(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        address = _parse_address(d.pop("address", UNSET))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_tariff(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        tariff = _parse_tariff(d.pop("tariff", UNSET))

        delivery_point_schema = cls(
            id=id,
            type_=type_,
            active=active,
            address=address,
            description=description,
            tariff=tariff,
        )

        delivery_point_schema.additional_properties = d
        return delivery_point_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from celine.sdk.auth.models import AccessToken
from celine.sdk.auth.provider import TokenProvider
from celine.sdk.auth.oidc_discovery import OidcDiscoveryClient, OidcConfiguration
from celine.sdk.auth.oidc import OidcClientCredentialsProvider

from celine.sdk.auth.jwt import JwtUser, is_service_account

__all__ = [
    "AccessToken",
    "TokenProvider",
    "OidcDiscoveryClient",
    "OidcConfiguration",
    "OidcClientCredentialsProvider",
    "JwtUser",
    "is_service_account",
]



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.generic_payload import GenericPayload


T = TypeVar("T", bound="SimulationDescriptorSchema")


@_attrs_define
class SimulationDescriptorSchema:
    """
    Attributes:
        key (str): Simulation key.
        meta (GenericPayload):
        description (None | str | Unset):
        title (None | str | Unset):
    """

    key: str
    meta: GenericPayload
    description: None | str | Unset = UNSET
    title: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        key = self.key

        meta = self.meta.to_dict()

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        title: None | str | Unset
        if isinstance(self.title, Unset):
            title = UNSET
        else:
            title = self.title

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "key": key,
                "meta": meta,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if title is not UNSET:
            field_dict["title"] = title

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.generic_payload import GenericPayload

        d = dict(src_dict)
        key = d.pop("key")

        meta = GenericPayload.from_dict(d.pop("meta"))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_title(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        title = _parse_title(d.pop("title", UNSET))

        simulation_descriptor_schema = cls(
            key=key,
            meta=meta,
            description=description,
            title=title,
        )

        simulation_descriptor_schema.additional_properties = d
        return simulation_descriptor_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.fetch_result_schema_items_item import FetchResultSchemaItemsItem


T = TypeVar("T", bound="FetchResultSchema")


@_attrs_define
class FetchResultSchema:
    """Result of a value fetch operation.

    Attributes:
        count (int):
        items (list[FetchResultSchemaItemsItem]):
        limit (int):
        offset (int):
    """

    count: int
    items: list[FetchResultSchemaItemsItem]
    limit: int
    offset: int

    def to_dict(self) -> dict[str, Any]:
        count = self.count

        items = []
        for items_item_data in self.items:
            items_item = items_item_data.to_dict()
            items.append(items_item)

        limit = self.limit

        offset = self.offset

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "count": count,
                "items": items,
                "limit": limit,
                "offset": offset,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.fetch_result_schema_items_item import FetchResultSchemaItemsItem

        d = dict(src_dict)
        count = d.pop("count")

        items = []
        _items = d.pop("items")
        for items_item_data in _items:
            items_item = FetchResultSchemaItemsItem.from_dict(items_item_data)

            items.append(items_item)

        limit = d.pop("limit")

        offset = d.pop("offset")

        fetch_result_schema = cls(
            count=count,
            items=items,
            limit=limit,
            offset=offset,
        )

        return fetch_result_schema



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.asset_relationships_in import AssetRelationshipsIn
    from ..models.device_in import DeviceIn


T = TypeVar("T", bound="StorageAssetIn")


@_attrs_define
class StorageAssetIn:
    """Battery storage asset.

    Attributes:
        name (str):
        battery_type (None | str | Unset):
        capacity (float | None | Unset):
        device (DeviceIn | None | Unset):
        installation_date (None | str | Unset):
        max_charge_power (float | None | Unset):
        max_discharge_power (float | None | Unset):
        relationships (AssetRelationshipsIn | Unset): Relationships between assets.
        round_trip_efficiency (float | None | Unset):
    """

    name: str
    battery_type: None | str | Unset = UNSET
    capacity: float | None | Unset = UNSET
    device: DeviceIn | None | Unset = UNSET
    installation_date: None | str | Unset = UNSET
    max_charge_power: float | None | Unset = UNSET
    max_discharge_power: float | None | Unset = UNSET
    relationships: AssetRelationshipsIn | Unset = UNSET
    round_trip_efficiency: float | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.device_in import DeviceIn

        name = self.name

        battery_type: None | str | Unset
        if isinstance(self.battery_type, Unset):
            battery_type = UNSET
        else:
            battery_type = self.battery_type

        capacity: float | None | Unset
        if isinstance(self.capacity, Unset):
            capacity = UNSET
        else:
            capacity = self.capacity

        device: dict[str, Any] | None | Unset
        if isinstance(self.device, Unset):
            device = UNSET
        elif isinstance(self.device, DeviceIn):
            device = self.device.to_dict()
        else:
            device = self.device

        installation_date: None | str | Unset
        if isinstance(self.installation_date, Unset):
            installation_date = UNSET
        else:
            installation_date = self.installation_date

        max_charge_power: float | None | Unset
        if isinstance(self.max_charge_power, Unset):
            max_charge_power = UNSET
        else:
            max_charge_power = self.max_charge_power

        max_discharge_power: float | None | Unset
        if isinstance(self.max_discharge_power, Unset):
            max_discharge_power = UNSET
        else:
            max_discharge_power = self.max_discharge_power

        relationships: dict[str, Any] | Unset = UNSET
        if not isinstance(self.relationships, Unset):
            relationships = self.relationships.to_dict()

        round_trip_efficiency: float | None | Unset
        if isinstance(self.round_trip_efficiency, Unset):
            round_trip_efficiency = UNSET
        else:
            round_trip_efficiency = self.round_trip_efficiency

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if battery_type is not UNSET:
            field_dict["battery_type"] = battery_type
        if capacity is not UNSET:
            field_dict["capacity"] = capacity
        if device is not UNSET:
            field_dict["device"] = device
        if installation_date is not UNSET:
            field_dict["installation_date"] = installation_date
        if max_charge_power is not UNSET:
            field_dict["max_charge_power"] = max_charge_power
        if max_discharge_power is not UNSET:
            field_dict["max_discharge_power"] = max_discharge_power
        if relationships is not UNSET:
            field_dict["relationships"] = relationships
        if round_trip_efficiency is not UNSET:
            field_dict["round_trip_efficiency"] = round_trip_efficiency

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.asset_relationships_in import AssetRelationshipsIn
        from ..models.device_in import DeviceIn

        d = dict(src_dict)
        name = d.pop("name")

        def _parse_battery_type(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        battery_type = _parse_battery_type(d.pop("battery_type", UNSET))

        def _parse_capacity(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        capacity = _parse_capacity(d.pop("capacity", UNSET))

        def _parse_device(data: object) -> DeviceIn | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                device_type_0 = DeviceIn.from_dict(data)

                return device_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DeviceIn | None | Unset, data)

        device = _parse_device(d.pop("device", UNSET))

        def _parse_installation_date(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        installation_date = _parse_installation_date(d.pop("installation_date", UNSET))

        def _parse_max_charge_power(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        max_charge_power = _parse_max_charge_power(d.pop("max_charge_power", UNSET))

        def _parse_max_discharge_power(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        max_discharge_power = _parse_max_discharge_power(d.pop("max_discharge_power", UNSET))

        _relationships = d.pop("relationships", UNSET)
        relationships: AssetRelationshipsIn | Unset
        if isinstance(_relationships, Unset):
            relationships = UNSET
        else:
            relationships = AssetRelationshipsIn.from_dict(_relationships)

        def _parse_round_trip_efficiency(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        round_trip_efficiency = _parse_round_trip_efficiency(d.pop("round_trip_efficiency", UNSET))

        storage_asset_in = cls(
            name=name,
            battery_type=battery_type,
            capacity=capacity,
            device=device,
            installation_date=installation_date,
            max_charge_power=max_charge_power,
            max_discharge_power=max_discharge_power,
            relationships=relationships,
            round_trip_efficiency=round_trip_efficiency,
        )

        storage_asset_in.additional_properties = d
        return storage_asset_in

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.lineage_facets_type_0 import LineageFacetsType0


T = TypeVar("T", bound="Lineage")


@_attrs_define
class Lineage:
    """
    Attributes:
        created_at (None | str | Unset):
        description (None | str | Unset):
        facets (LineageFacetsType0 | None | Unset):
        last_lifecycle_state (None | str | Unset):
        last_modified_at (None | str | Unset):
        name (None | str | Unset):
        namespace (None | str | Unset):
        source_name (None | str | Unset):
        tags (list[str] | None | Unset):
        title (None | str | Unset):
        updated_at (None | str | Unset):
    """

    created_at: None | str | Unset = UNSET
    description: None | str | Unset = UNSET
    facets: LineageFacetsType0 | None | Unset = UNSET
    last_lifecycle_state: None | str | Unset = UNSET
    last_modified_at: None | str | Unset = UNSET
    name: None | str | Unset = UNSET
    namespace: None | str | Unset = UNSET
    source_name: None | str | Unset = UNSET
    tags: list[str] | None | Unset = UNSET
    title: None | str | Unset = UNSET
    updated_at: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.lineage_facets_type_0 import LineageFacetsType0

        created_at: None | str | Unset
        if isinstance(self.created_at, Unset):
            created_at = UNSET
        else:
            created_at = self.created_at

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        facets: dict[str, Any] | None | Unset
        if isinstance(self.facets, Unset):
            facets = UNSET
        elif isinstance(self.facets, LineageFacetsType0):
            facets = self.facets.to_dict()
        else:
            facets = self.facets

        last_lifecycle_state: None | str | Unset
        if isinstance(self.last_lifecycle_state, Unset):
            last_lifecycle_state = UNSET
        else:
            last_lifecycle_state = self.last_lifecycle_state

        last_modified_at: None | str | Unset
        if isinstance(self.last_modified_at, Unset):
            last_modified_at = UNSET
        else:
            last_modified_at = self.last_modified_at

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        namespace: None | str | Unset
        if isinstance(self.namespace, Unset):
            namespace = UNSET
        else:
            namespace = self.namespace

        source_name: None | str | Unset
        if isinstance(self.source_name, Unset):
            source_name = UNSET
        else:
            source_name = self.source_name

        tags: list[str] | None | Unset
        if isinstance(self.tags, Unset):
            tags = UNSET
        elif isinstance(self.tags, list):
            tags = self.tags

        else:
            tags = self.tags

        title: None | str | Unset
        if isinstance(self.title, Unset):
            title = UNSET
        else:
            title = self.title

        updated_at: None | str | Unset
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = self.updated_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if description is not UNSET:
            field_dict["description"] = description
        if facets is not UNSET:
            field_dict["facets"] = facets
        if last_lifecycle_state is not UNSET:
            field_dict["lastLifecycleState"] = last_lifecycle_state
        if last_modified_at is not UNSET:
            field_dict["lastModifiedAt"] = last_modified_at
        if name is not UNSET:
            field_dict["name"] = name
        if namespace is not UNSET:
            field_dict["namespace"] = namespace
        if source_name is not UNSET:
            field_dict["sourceName"] = source_name
        if tags is not UNSET:
            field_dict["tags"] = tags
        if title is not UNSET:
            field_dict["title"] = title
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.lineage_facets_type_0 import LineageFacetsType0

        d = dict(src_dict)

        def _parse_created_at(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        created_at = _parse_created_at(d.pop("createdAt", UNSET))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_facets(data: object) -> LineageFacetsType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                facets_type_0 = LineageFacetsType0.from_dict(data)

                return facets_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(LineageFacetsType0 | None | Unset, data)

        facets = _parse_facets(d.pop("facets", UNSET))

        def _parse_last_lifecycle_state(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        last_lifecycle_state = _parse_last_lifecycle_state(d.pop("lastLifecycleState", UNSET))

        def _parse_last_modified_at(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        last_modified_at = _parse_last_modified_at(d.pop("lastModifiedAt", UNSET))

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_namespace(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        namespace = _parse_namespace(d.pop("namespace", UNSET))

        def _parse_source_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        source_name = _parse_source_name(d.pop("sourceName", UNSET))

        def _parse_tags(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                tags_type_0 = cast(list[str], data)

                return tags_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        tags = _parse_tags(d.pop("tags", UNSET))

        def _parse_title(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        title = _parse_title(d.pop("title", UNSET))

        def _parse_updated_at(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        updated_at = _parse_updated_at(d.pop("updatedAt", UNSET))

        lineage = cls(
            created_at=created_at,
            description=description,
            facets=facets,
            last_lifecycle_state=last_lifecycle_state,
            last_modified_at=last_modified_at,
            name=name,
            namespace=namespace,
            source_name=source_name,
            tags=tags,
            title=title,
            updated_at=updated_at,
        )

        lineage.additional_properties = d
        return lineage

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
